package Nada::Zip::Bupkes;
#                                doom@kzsu.stanford.edu
#                                26 Apr 2007

use 5.006;
use strict; 
use warnings;
my $DEBUG = 1;
use Carp;
use Data::Dumper;

our $VERSION = '0.01';

# Object code goes here.



1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

Nada::Zip::Bupkes - Perl extension for blah blah blah

=head1 SYNOPSIS

   use Nada::Zip::Bupkes;
   blah blah blah

=head1 DESCRIPTION

Stub documentation for Nada::Zip::Bupkes, 
created by perlnow.el using template.el.

It looks like the author of the extension was negligent 
enough to leave the stub unedited.

Blah blah blah.

=head2 EXPORT

None by default.

=head1 SEE ALSO

Mention other useful documentation such as the documentation of
related modules or operating system documentation (such as man pages
in UNIX), or any relevant external documentation such as RFCs or
standards.

If you have a mailing list set up for your module, mention it here.

If you have a web site set up for your module, mention it here.

=head1 AUTHOR

Joseph Brenner, E<lt>doom@kzsu.stanford.eduE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 by Joseph Brenner

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.2 or,
at your option, any later version of Perl 5 you may have available.

=head1 BUGS

None reported... yet.

=cut
